@@ -1,4 +1,4 @@
 .onLoad <- function(lib, pkg) {
-	cat("DatABEL v 0.1-6 (June 21, 2010) loaded\n")
+	cat("DatABEL v 0.9-0 (August 04, 2010) loaded\n")
 	.Call("checkNumBits");	
 }
\ No newline at end of file
